Skip to contents

Simulated fine-mapping data set used to illustrate mvSuSiE. Includes genotype and phenotype data for 574 samples, 1,001 genetic markers and 20 traits. The traits were simulated from the mvSuSiE model with coefficients simdata$Btrue and residual covariance simdata$par$V. Three causal genetic variants at positions 255, 335 and 493 have nonzero coefficients.

Format

simdata is a list with the following elements:

raw$X

The matrix of simulated genotypes.

raw$Y

The matrix of simulated traits.

Btrue

The coefficients used to simulate the data.

par$V

The residual covariance matrix used to simulate the data.

par$U

The collection of covariance matrices specifying the mvsusie prior.

par$w

The weights associated with the covariance matrices.

sumstats$n

The sample size.

sumstats$LD

The LD computed from raw$X.

sumstats$bhat

The least-squares effect estimates from the single-marker association tests computed using univariate_regression. (Note that X was standardized before computing bhat.)

sumstats$sehat

The standard errors of the least-squares effect estimates computed using univariate_regression. (Note that X was standardized before computing sehat.)